/******************************************************************************/
/* FILE: HS.C  source for HOSTESS S controller diagnostic and examples        */
/* Release 1.00                                                               */
/* Date 18 December 1992                                                      */
/* Copyright (C) 1992 Comtrol Corporation                                     */
/******************************************************************************/

/* header files */
#include <stdio.h>
#include <dos.h>
#include <conio.h>
#include "hs.h"

/* function prototypes */
void Set_Reg_Addr(void);
void Tx_1_DMA1(void);
void Tx_1_DMA3(void);
void Rx_1_Polled(void);
void Tx_1_Polled(void);
void Tx_2_Polled(void);
void Tx_2_DMA3(void);
void AutoTest(void);
void SCC_Init(int Chan_Cmd_Reg);
void SCC_Enable(int Chan_Cmd_Reg);
int  Data_Init(void);
void Rx_2_Polled(void);
int  Test_1_2(void);
void Sys_DMA_Init(int Channel, int Mode, unsigned long Abs_Address, int Count);
void SCC_DMA_Init(int Chan_Cmd_Reg, int Mode);
void Test_IRQ(void);
void ModemDB25(void);
void ModemDB9(void);
int  MainMenu(void);
int  BaudMenu(void);
int  BaseMenu(void);
int  DmaMenu(void);
int  IRQMenu(void);
void wait(long int wtime);
void calibrate_wait(void);
void Init_Bisync(int Chan_Cmd_Reg, int ClockMode);

/* our interrupt handling routine */
void hndirq( void );

/******************************************************************************/
/* main: displays menus, calls selected test routines.                        */
/******************************************************************************/
void main(void)
{
   int i;
   unsigned long addr;                       /* temporary address storage */

   /* compute absolute address of sys_dat_rx[] and sys_data_tx[] for DMA */
   addr = (unsigned long)sys_data_rx;
   sys_rx_abs = ((addr & 0xFFFF0000) >> 12) + (addr & 0x0000FFFF);
   addr = (unsigned long)sys_data_tx;
   sys_tx_abs = ((addr & 0xFFFF0000) >> 12) + (addr & 0x0000FFFF);

   BaudIndex = 0;      /* baud index default: 19200 baud index */
   IRQ = 5;            /* IRQ menu choice default: IRQ 5  */
   Base = 0x200;       /* Base menu choice default: I/O base address 200 HEX */

   Set_Reg_Addr();     /* set up default register addresses */

   calibrate_wait();   /* calibrate wait function for dynamic delays */

   while(MainChoice != QUIT)   /* clear scrn, print settings, put up menu */
   {
      MainChoice = MainMenu();
      switch(MainChoice)
      {
         case SETBASE:
            system("cls");
            Base = BaseMenu();
            Set_Reg_Addr();
            break;
         case SETIRQ:
            system("cls");
            IRQ = IRQMenu();
            break;
         case SETBAUD:
            system("cls");
            BaudIndex = BaudMenu();
            break;
         case R1_POLLED:
            Rx_1_Polled();
            break;
         case R2_POLLED:
            Rx_2_Polled();
            break;
         case X1_POLLED:
            Tx_1_Polled();
            break;
         case X2_POLLED:
            Tx_2_Polled();
            break;
         case X1_DMA1:
            Tx_1_DMA1();
            break;
         case X2_DMA3:
            Tx_2_DMA3();
            break;
         case X1_DMA3:
            Tx_1_DMA3();
            break;
         case AUTO:
            AutoTest();
            break;
         case QUIT:
            break;
         default:
            break;
      }
   }
   system("cls");
   exit(0);
}

/******************************************************************************/
/* name: Data_Init()                                                          */
/* entry: none                                                                */
/* function: loads up memory with Tx data (global sys_data_tx array).         */
/* exit: returns number of bytes loaded into memory.                          */
/******************************************************************************/
int Data_Init(void)
{
   int count, k;                    /* total byte count */
   char i, j;                       /* character counters */

   count = 0;
   for(j = 0; j <= 47; j++)         /* load memory with one screen of data */
   {                                /* 80 by 48 chars */
      for(i = 0x30; i <= 0x57; i++)
      {
         sys_data_tx[i - 0x30 + (j * 0x28)] = i; /* set up 1/2 line at a time */
         count++;
      }
   }

   for(k = 0; k <= count; k++)      /* clear Rx data from memory */
      sys_data_rx[k] = 0;

   return(count);
}

/******************************************************************************/
/* name: SCC_Init()                                                           */
/* entry: channel command register address is passed by calling function.     */
/*        baud rate is in global array baud[].                                */
/* function: reset SCC channel, initialize the SCC registers.                 */
/******************************************************************************/
void SCC_Init(int Chan_Cmd_Reg)
{
   int temp;   /* bit manipulation */

   /* reset the desired channel (channel reset is common register) */
   outp(Chan_Cmd_Reg, WR9);                    /* set index reg for wr9 */
   if(Chan_Cmd_Reg == chan_a_cmd)              /* init chan a ? */
      outp(Chan_Cmd_Reg, RESET_CHA);           /* reset channel a */
   if(Chan_Cmd_Reg == chan_b_cmd)              /* init chan b ? */
      outp(Chan_Cmd_Reg, RESET_CHB);           /* reset channel b */

   /* scc initialization */
   outp(Chan_Cmd_Reg, WR4);                    /* set index reg for wr4 */
   outp(Chan_Cmd_Reg, RXTX_PARMS);             /* x16 clock, 1stop, no parity */
   outp(Chan_Cmd_Reg, WR3);                    /* set index reg for wr3 */
   outp(Chan_Cmd_Reg, RX_8BITS_CHAR);          /* 8 bits per char rcv */
   outp(Chan_Cmd_Reg, WR5);                    /* set index reg for wr5 */
   outp(Chan_Cmd_Reg, TX_8BITS_CHAR);          /* 8 bits per char xmit */
   outp(Chan_Cmd_Reg, WR11);                   /* set index reg for wr11*/
   outp(Chan_Cmd_Reg, CLK_SOURCE);             /* rx and tx clock = brg */
   brtc_lsb = (char)(baud[BaudIndex] & 0xFF);
   outp(Chan_Cmd_Reg, WR12);                   /* set index reg for wr12*/
   outp(Chan_Cmd_Reg, brtc_lsb);               /* baud rate const LSB */
   temp = (baud[BaudIndex] >> 8) & 0xFF;       /* calc MSB of brtc */
   brtc_msb = (char)temp;
   outp(Chan_Cmd_Reg, WR13);                   /* set index reg for wr13*/
   outp(Chan_Cmd_Reg, brtc_msb);               /* baud rate const MSB */
   outp(Chan_Cmd_Reg, WR14);                   /* set index reg for wr14*/
   outp(Chan_Cmd_Reg, BRG_EQ_SYS_CLK);         /* brg source = sys clock */
}

/******************************************************************************/
/* name: SCC_Enable()                                                         */
/* entry: SCC channel command register address is passed by calling function. */
/* function: enable Tx, Rx and baud rate generator (brg) for polled operation.*/
/******************************************************************************/
void SCC_Enable(int Chan_Cmd_Reg)
{
   /* scc enable */
   outp(Chan_Cmd_Reg, WR15);                      /* set index reg for wr15 */
   outp(Chan_Cmd_Reg, 0x0);                       /* disable interrupts */
   outp(Chan_Cmd_Reg, WR14);                      /* set index reg for wr14*/
   outp(Chan_Cmd_Reg, BRG_EQ_SYS_CLK|BRG_ENABLE); /* enable brg */
   outp(Chan_Cmd_Reg, WR3);                       /* set index reg for wr3 */
   outp(Chan_Cmd_Reg, RX_8BITS_CHAR|RX_ENABLE);   /* enable rx */
   outp(Chan_Cmd_Reg, WR5);                       /* set index reg for wr5 */
   outp(Chan_Cmd_Reg, TX_8BITS_CHAR|TX_ENABLE);   /* enable tx */
}

/******************************************************************************/
/* name: SCC_DMA_Init()                                                       */
/* entry: SCC channel command register address is passed by calling function. */
/*        SCC DMA Mode is passed by calling function.                         */
/* function: enable Tx, Rx and baud rate generator (brg) for dma operation.   */
/******************************************************************************/
void SCC_DMA_Init(int Chan_Cmd_Reg, int Mode)
{
   switch(Mode)
   {
      case WREQ_RX:  /* SCC initialization for DMA w/req on receive */
         outp(Chan_Cmd_Reg, WR1);                        /* index wr1 */
         outp(Chan_Cmd_Reg, RXDMA_W);                    /* dma w/req on rcv */
         outp(Chan_Cmd_Reg, WR14);                       /* index wr14 */
         outp(Chan_Cmd_Reg, BRG_EQ_SYS_CLK|BRG_ENABLE);  /* enable brg */
         break;

      case WREQ_TX:  /* SCC initialization for DMA w/req on transmit */
         outp(Chan_Cmd_Reg, WR1);                        /* index wr1 */
         outp(Chan_Cmd_Reg, TXDMA_W);                    /* dma w/req on Tx */
         outp(Chan_Cmd_Reg, WR14);                       /* index wr14*/
         outp(Chan_Cmd_Reg, BRG_EQ_SYS_CLK|BRG_ENABLE);  /* enable brg */
         break;

      case DTRQ_TX:  /* SCC initialization for DMA dtr/req on transmit */
         outp(Chan_Cmd_Reg, WR14);                       /* index wr14 */
         outp(Chan_Cmd_Reg, BRG_EQ_SYS_CLK | TXDMA_DTR); /* DTR/req enable */
         outp(Chan_Cmd_Reg, WR14);                       /* index wr14 */
         outp(Chan_Cmd_Reg, BRG_EQ_SYS_CLK|BRG_ENABLE|TXDMA_DTR);
         break;                                          /* enable brg */

      default:
         break;
   }
   outp(Chan_Cmd_Reg, WR3);                         /* set index reg for wr3 */
   outp(Chan_Cmd_Reg, RX_8BITS_CHAR | RX_ENABLE);   /* enable rx */
   outp(Chan_Cmd_Reg, WR5);                         /* set index reg for wr5 */
   outp(Chan_Cmd_Reg, TX_8BITS_CHAR | TX_ENABLE);   /* enable tx */
}


/******************************************************************************/
/* name: Sys_DMA_Init()                                                       */
/* entry: calling function provides: Mode (read or write)                     */
/*                                   Count (byte count)                       */
/*                                   Abs_Address (absolute address of data)   */
/*                                   Channel (DMA channel to set up)          */
/* function: set up SYSTEM DMA controller.                                    */
/* *** NOTE: This function sets up the DMA controller on the PC motherboard   */
/******************************************************************************/
void Sys_DMA_Init(int Channel, int Mode, unsigned long Abs_Address, int Count)
{
   int temp;        /* bit manipulation */

   switch( Mode )
   {
      case WRITE:                   /* write transfer = DMA from I/O to mem */
         switch( Channel )
         {
            case 1:
               outp(WR_MODE, WRITE_XFER | SINGLE_MODE | ADDR_INC | CH1_SEL);
               break;
            case 3:
               outp(WR_MODE, WRITE_XFER | SINGLE_MODE | ADDR_INC | CH3_SEL);
               break;
         }
         break;
      case READ:                    /* read transfer = DMA from memory to I/O */
         switch( Channel )
         {
            case 1:
               outp(WR_MODE, AUTO_INIT|READ_XFER|SINGLE_MODE|ADDR_INC|CH1_SEL);
               break;
            case 3:
               outp(WR_MODE, AUTO_INIT|READ_XFER|SINGLE_MODE|ADDR_INC|CH3_SEL);
               break;
         }
         break;
      default:
         break;
   }
   outp(CLR_PNTR, 0x0); /* clear byte pointer F/F, any value resets the F/F */

   switch( Channel )
   {
      case 1:
         temp = (Abs_Address >> 16) & 0x0F;  /* page register (bits A17-A20) */
         outp(DMA1_PG_REG, (char)temp);      /* write page register channel 1 */
         temp = Abs_Address & 0xff;          /* calc base address low byte */
         outp(CH1_ADD, (char)temp);          /* write address low byte */
         temp = (Abs_Address >> 8) & 0xff;   /* calc base address high byte */
         outp(CH1_ADD, (char)temp);          /* write address high byte */
         temp = Count & 0xFF;                /* calc byte count low byte */
         outp(CH1_BC, (char)temp);           /* write byte count low byte */
         temp = (Count >> 8) & 0xFF;         /* calc byte count high byte */
         outp(CH1_BC, (char)temp);           /* write byte count high byte */
         break;
      case 3:
         temp = (Abs_Address >> 16) & 0x0F;  /* page register (bits A17-A20) */
         outp(DMA3_PG_REG, (char)temp);      /* write page register channel 3 */
         temp = Abs_Address & 0xff;          /* calc base address low byte */
         outp(CH3_ADD, (char)temp);          /* write address low byte */
         temp = (Abs_Address >> 8) & 0xff;   /* calc base address high byte */
         outp(CH3_ADD, (char)temp);          /* write address high byte */
         temp = Count & 0xFF;                /* calc byte count low byte */
         outp(CH3_BC, (char)temp);           /* write byte count low byte */
         temp = (Count >> 8) & 0xFF;         /* calc byte count high byte */
         outp(CH3_BC, (char)temp);           /* write byte count high byte */
         break;
      default:
         break;
   }
}

/******************************************************************************/
/* name: Tx_1_Polled()                                                      */
/* entry: none                                                                */
/* function: transmit from port a using polling method.                       */
/******************************************************************************/
void Tx_1_Polled(void)
{
   int temp;                    /* bit manipulation holder */
   char i, status;              /* character counter, status reg holder */

   outp(config_reg, DTR_A_OFF); /* set HOSTESS S config reg options off */
   SCC_Init(chan_a_cmd);        /* initialize SCC */
   SCC_Enable(chan_a_cmd);      /* enable SCC polled mode */

   /* clear screen and print message */
   system("cls");
   printf("       Transmitting chars out port a polled.\n");
   printf("\n\n   Press any key to exit.");       /* any key msg */
   while(!kbhit())                                /* keep outputting chars */
   {                                              /* until the user hits a */
      for(i = 0x30; i <= 0x57; i++)               /* key to terminate */
      {
         outp(chan_a_cmd, RR0);           /* set up for status register (RR0) */
         while(!(inp(chan_a_cmd) & 0x4))  /* check for TX buffer empty */
         {
            if(kbhit())
               break;
            outp(chan_a_cmd, RR0);        /* keep checking RR0 */
         }
         outp(chan_a_data, i);            /* output data to scc */
      }
   }
   getche();         /* clear out the keyboard buffer */
}

/******************************************************************************/
/* name: Rx_1_Polled()                                                       */
/* entry: none                                                                */
/* function: receives chars from port a using polling method.                 */
/*                                                                            */
/******************************************************************************/
void Rx_1_Polled(void)
{
   int temp;                        /* bit manipulation holder */
   char i, status;                  /* character counter, status reg holder */
   char rx_char;                    /* receive data holder */

   outp(config_reg, DTR_A_OFF); /* set HOSTESS S config reg options off */
   SCC_Init(chan_a_cmd);        /* initialize and enable SCC */
   SCC_Enable(chan_a_cmd);      /* enable polled mode */

   /* clear screen and print */
   system("cls");
   printf("       Polling port a for received chars.\n");
   printf("\n\n   Press any key to exit.\n\n"); /* any key msg */

   while(!kbhit())                     /* keep reading chars */
   {                                   /* until the user hits a key */
      outp(chan_a_cmd, RR0);           /* set up for status register (RR0) */
      while(!(inp(chan_a_cmd) & 0x1))  /* check for RX buffer full */
      {
         if(kbhit())
            break;
         outp(chan_a_cmd, RR0);        /* keep checking RR0 */
      }
      rx_char = inp(chan_a_data);      /* get data from scc */
      printf("%c",rx_char);            /* print character to console */
   }
   getche();                           /* clear out the keyboard buffer */
}

/******************************************************************************/
/* name: Rx_2_Polled()                                                       */
/* entry: none                                                                */
/* function: receives chars from port 2 using polling method.                 */
/*                                                                            */
/******************************************************************************/
void Rx_2_Polled(void)
{
   int temp;                    /* bit manipulation holder */
   char i, status;              /* character counter, status reg holder */
   char rx_char;                /* receive data holder */

   outp(config_reg, DTR_A_OFF); /* set HOSTESS S config reg options off */
   SCC_Init(chan_b_cmd);        /* initialize and enable SCC */
   SCC_Enable(chan_b_cmd);      /* enable polled mode */

   /* clear screen and print */
   system("cls");
   printf("       Polling port b for received chars.\n");
   printf("\n\n   Press any key to exit.\n\n"); /* any key msg */

   while(!kbhit())                     /* keep reading chars */
   {                                   /* until the user hits a key */
      outp(chan_b_cmd, RR0);           /* set up for status register (RR0) */
      while(!(inp(chan_b_cmd) & 0x1))  /* check for RX buffer full */
      {
         if(kbhit())
            break;
         outp(chan_b_cmd, RR0);        /* keep checking RR0 */
      }
      rx_char = inp(chan_b_data);      /* get data from scc */
      printf("%c",rx_char);            /* print character to console */
   }
   getche();                           /* clear out the keyboard buffer */
}

/******************************************************************************/
/* name: Tx_2_Polled()                                                        */
/* entry: none                                                                */
/* function: transmit from port 2 using polling method.                       */
/******************************************************************************/
void Tx_2_Polled(void)
{
   int temp;                    /* bit manipulation holder */
   char i, status;              /* character counter, status reg holder */

   outp(config_reg, DTR_A_OFF); /* set HOSTESS S config reg options off */
   SCC_Init(chan_b_cmd);        /* initialize SCC */
   SCC_Enable(chan_b_cmd);      /* enable SCC for polling */

   /* clear console and print any key msg */
   system("cls");
   printf("       Transmitting chars out port b polled.\n");
   printf("\n\n   Press any key to exit.");

   while(!kbhit())                  /* keep outputting chars until */
   {                                /* the user hits a key to terminate */
      for(i = 0x30; i <= 0x57; i++) /* ascii chars 0x30 (0) to 0x57 (W) */
      {
         outp(chan_b_cmd, RR0);           /* set up for status register (RR0) */
         while(!(inp(chan_b_cmd) & 0x4))  /* check for TX buffer empty */
         {
            if(kbhit())
               break;
            outp(chan_b_cmd, RR0);        /* keep checking RR0 */
         }
         outp(chan_b_data, i);            /* output data to scc */
      }
   }
   getche();                              /* clear out the keyboard buffer */
}

/******************************************************************************/
/* name: Tx_1_DMA1()                                                          */
/* entry: none                                                                */
/* function: transmit characters from port 1 using DMA channel 1 method.      */
/******************************************************************************/
void Tx_1_DMA1(void)
{
   int temp, byte_count;                /* bit manipulation, DMA word count */
   char status;                         /* status reg holder */

   byte_count = Data_Init();            /* load chars into memory for Tx */

   /* ask which SCC DMA type to use (W/REQ or DTR/REQ) */
   switch(DmaMenu())
   {
      case W_REQ:
         /* DMA channel selection, sets up for W/REQ on port a, dma chan 1 */
         outp(config_reg, DMA1_W | DTR_A_OFF);
         SCC_Init(chan_a_cmd);              /* initialize SCC com parms */
         SCC_DMA_Init(chan_a_cmd, WREQ_TX); /* initialize SCC DMA parms */
         break;
      case DTR_REQ:
         /* DMA channel selection, sets up for DTR/REQ on port a, dma chan 1 */
         outp(config_reg, DMA1_DTR | DTR_A_OFF); /* sets HOSTESS S for DRQ1 on channel 1 */
         SCC_Init(chan_a_cmd);                   /* initialize SCC */
         SCC_DMA_Init(chan_a_cmd, DTRQ_TX);      /* initialize SCC DMA parms */
         break;
      default:
         break;
   }

   Sys_DMA_Init( 1, READ, sys_tx_abs, byte_count); /* DMA controller setup */
   outp(WR_SNGL_MSK, 0x1);                        /* unmask chan 1, start dma */

   system("cls");
   printf("Transmitting chars from port a, DMA channel 1\n");
   printf("Press any key to stop.\n");
   while(!kbhit());    /* wait for key to be pressed */
                       /* do nothing while waiting, DMA runs in background */
   getche();           /* get rid of key in buffer */

   outp(WR_SNGL_MSK, CH1_OFF); /* mask off the DMA request (stop DMA xfers) */
   outp(CLR_PNTR, 0x0);       /* clear byte ptr F/F, any value resets the F/F */
}

/******************************************************************************/
/* name: Tx_1_DMA3()                                                          */
/* entry: none                                                                */
/* function: transmit from port a using DMA channel 3 method.                 */
/******************************************************************************/
void Tx_1_DMA3(void)
{
   int temp, byte_count;            /* bit manipulation, DMA ctrlr byte count */
   char status;                     /* status reg holder */

   byte_count = Data_Init();        /* load chars into memory for Tx */

   /* HOSTESS S config register initialization for DMA */
   switch(DmaMenu())   /* ask which SCC DMA type to use (W/REQ or DTR/REQ) */
   {
      case W_REQ:
         /* DMA channel selection, sets up for W/REQ on port a, dma chan 3 */
         outp(config_reg, DMA3_W | DTR_A_OFF);
         SCC_Init(chan_a_cmd);              /* initialize SCC */
         SCC_DMA_Init(chan_a_cmd, WREQ_TX); /* initialize SCC DMA parms */
         break;
      case DTR_REQ:
         /* DMA channel selection, sets up for DTR/REQ on port a, dma chan 3 */
         outp(config_reg, DMA3_DTR | DTR_A_OFF); /* DRQ3 on chan a */
         SCC_Init(chan_a_cmd);                   /* initialize SCC */
         SCC_DMA_Init(chan_a_cmd, DTRQ_TX);      /* initialize SCC DMA parms */
         break;
      default:
         break;
   }

   Sys_DMA_Init( 3, READ, sys_tx_abs, byte_count); /* DMA controller setup */
   outp(WR_SNGL_MSK, 0x3);                  /* unmask chan 3 (start DMA) */

   system("cls");
   printf("Transmitting chars from port a, DMA channel 3.");
   while(!kbhit());          /* wait for key to be pressed */
                             /* do nothing, DMA transfers in background */
   getche();                 /* get rid of key in buffer */
   outp(WR_SNGL_MSK, CH3_OFF);   /* mask off the DMA request line (stop DMA) */
   outp(CLR_PNTR, 0x0);      /* clear byte ptr F/F, any value resets the F/F */
}

/******************************************************************************/
/* name: Tx_2_DMA3()                                                          */
/* entry: nothing                                                             */
/* function: transmit characters from port2 using DMA channel 3 method.       */
/******************************************************************************/
void Tx_2_DMA3(void)
{
   int temp, byte_count;            /* bit manipulation, DMA ctlr byte count */
   char status;                     /* status reg holder */

   byte_count = Data_Init();        /* load chars into memory for Tx */

   /* HOSTESS S config register initialization */
   /* DMA channel selection, sets up for W/REQ on port b, dma chan 3 */
   outp(config_reg, DMA3_B_W | DTR_A_OFF);

   SCC_Init(chan_b_cmd);              /* initialize SCC com parms */
   SCC_DMA_Init(chan_b_cmd, WREQ_TX); /* initialize SCC DMA parms */

   Sys_DMA_Init( 3, READ, sys_tx_abs, byte_count);   /* DMA controller setup */
   outp(WR_SNGL_MSK, 0x3);            /* unmask chan 3 (start DMA) */

   system("cls");
   printf("Transmitting chars from port b, DMA channel 3, using W/REQ");

   while(!kbhit());    /* wait for key to be pressed before clearing DMA */
                       /* do nothing while waiting, DMA in background */
   getche();           /* get rid of key in buffer */

   outp(WR_SNGL_MSK, CH3_OFF);   /* mask off the DMA3 request line (stop DMA) */
   outp(CLR_PNTR, 0x0);      /* clear byte pntr F/F, any value */
}

/******************************************************************************/
/* name: AutoTest()                                                           */
/* entry: none                                                                */
/* function: test controller completely                                       */
/******************************************************************************/
void AutoTest(void)
{
   int i;

   system("cls");

printf("ͻ");
printf("                                AUTO TEST                                     ");
printf("ͼ");


   printf("\nInstall TP020 test cable between Ports a and b DB25 connectors.");
   printf("\n           <PRESS ANY KEY WHEN READY>\n");
   getch();

   Test_1_2();   /* async, dma and sync tests, using loop between a and b */
   ModemDB25();  /* modem control line tests between port a and b */

   printf("\n\nRemove TP020 test cable, ");
   printf("install 24115 plug into Port b DB9 connector.");
   /* NOTE: pin-outs for cable and plug are in README.HS file */

   printf("\n           <PRESS ANY KEY WHEN READY>\n");
   getch();

   ModemDB9();          /* test Modem control lines during port 2 loopback */
   Test_IRQ();          /* test IRQ during port 2 loopback */

   printf("\n\n\nTesting complete, press any key to continue.......");
   getche();
}

/******************************************************************************/
/* name: ModemDB25()                                                          */
/* entry: none                                                                */
/* function: modem control signal test for loopback cable (port 1 to port 2). */
/******************************************************************************/
void ModemDB25(void)
{
   int i;                         /* counter */
   int failed = FALSE;            /* true if test fails */
   int SCCModemA, SCCModemB;      /* RR0 modem status read */
   int HSModemA, HSModemB;        /* Hostess S modem_stat register read */

   printf("\nPort to port modem control test ");

   outp(config_reg, DTR_A_OFF);   /* clear Hostess S config register */
   SCC_Init(chan_a_cmd);          /* initialize port a SCC */
   SCC_Enable(chan_a_cmd);        /* enable port a SCC */
   SCC_Init(chan_b_cmd);          /* initialize port b SCC */
   SCC_Enable(chan_b_cmd);        /* enable port b SCC */

   for(i=0; i<=10; i++)           /* test all combinations 10 times */
   {
      /***********************************/
      /* clear both ports' DTR and RTS   */
      /***********************************/
      outp(chan_a_cmd,WR5);                         /* index WR5 */
      outp(chan_a_cmd,TX_8BITS_CHAR);               /* RTS low */
      outp(config_reg, DTR_A_OFF);                  /* DTR low port a */
      outp(chan_b_cmd,WR5);                         /* index WR5 */
      outp(chan_b_cmd,TX_8BITS_CHAR);               /* DTR low, RTS low port b*/
      wait(4L);                                     /* 4 ms for propagation */
      HSModemA = ~(inp(modem_stat)) & HS_MDM_A;     /* Hostess S mdm inverted */
      HSModemB = ~(inp(modem_stat)) & HS_MDM_B;     /* Hostess S mdm inverted */
      outp(chan_a_cmd,RR0);                         /* index RR0 */
      SCCModemA = inp(chan_a_cmd) & SCC_MDM;        /* SCC port a modem stat */
      outp(chan_b_cmd,RR0);                         /* index RR0 */
      SCCModemB = inp(chan_b_cmd) & SCC_MDM;        /* SCC port b modem stat */
      if(SCCModemA || HSModemA || SCCModemB || HSModemB) {
         printf("...................FAILED!!!");
         failed = TRUE;                             /* all inputs unasserted */
         break;
      }
      /*********************************************/
      /* check DTR port a to DSR and DCD on port b */
      /*********************************************/
      outp(chan_a_cmd,WR5);                         /* index WR5 */
      outp(chan_a_cmd,TX_8BITS_CHAR);               /* RTS low */
      outp(config_reg, 0x0);                        /* Hostess S port a DTR */
      outp(chan_b_cmd,WR5);                         /* index WR5 */
      outp(chan_b_cmd,TX_8BITS_CHAR);               /* DTR low, RTS low */
      wait(4L);                                     /* 4 ms for propagation */
      HSModemA = ~(inp(modem_stat)) & HS_MDM_A;     /* Hostess S mdm inverted */
      HSModemB = ~(inp(modem_stat)) & HS_MDM_B;     /* Hostess S mdm inverted */
      outp(chan_a_cmd,RR0);                         /* index RR0 */
      SCCModemA = inp(chan_a_cmd) & SCC_MDM;        /* SCC port a modem stat */
      outp(chan_b_cmd,RR0);                         /* index RR0 */
      SCCModemB = inp(chan_b_cmd) & SCC_MDM;        /* SCC port b modem stat */
      if(!( (SCCModemB == DCD_) && (HSModemB == DSR2) )
             || SCCModemA || HSModemA) {
         printf("...................FAILED!!!");
         failed = TRUE;                             /* only port b DSR & DCD */
         break;
      }
      /*********************************************/
      /* check RTS port a to CTS and RI on port b  */
      /*********************************************/
      outp(chan_a_cmd,WR5);                         /* index WR5 */
      outp(chan_a_cmd,TX_8BITS_CHAR | RTS);         /* RTS high */
      outp(config_reg, DTR_A_OFF);                  /* port a DTR low */
      outp(chan_b_cmd,WR5);                         /* index WR5 */
      outp(chan_b_cmd,TX_8BITS_CHAR);               /* DTR low, RTS low */
      wait(4L);                                     /* 4 ms for propagation */
      HSModemA = ~(inp(modem_stat)) & HS_MDM_A;     /* Hostess S mdm inverted */
      HSModemB = ~(inp(modem_stat)) & HS_MDM_B;     /* Hostess S mdm inverted */
      outp(chan_a_cmd,RR0);                         /* index RR0 */
      SCCModemA = inp(chan_a_cmd) & SCC_MDM;        /* SCC port a modem stat */
      outp(chan_b_cmd,RR0);                         /* index RR0 */
      SCCModemB = inp(chan_b_cmd) & SCC_MDM;        /* SCC port b modem stat */
      if(!( (SCCModemB == CTS_) && (HSModemB == RI2) )
             || SCCModemA || HSModemA) {
         printf("...................FAILED!!!");
         failed = TRUE;                             /* only port b CTS & RI */
         break;
      }
      /*********************************************/
      /* check DTR port b to DSR and DCD on port a */
      /*********************************************/
      outp(chan_a_cmd,WR5);                         /* index WR5 */
      outp(chan_a_cmd,TX_8BITS_CHAR);               /* RTS low */
      outp(config_reg, DTR_A_OFF);                  /* port a DTR unasserted */
      outp(chan_b_cmd,WR5);                         /* index WR5 */
      outp(chan_b_cmd,TX_8BITS_CHAR | DTR);         /* DTR high, RTS low */
      wait(4L);                                     /* 4 ms for propagation */
      HSModemA = ~(inp(modem_stat)) & HS_MDM_A;     /* Hostess S mdm inverted */
      HSModemB = ~(inp(modem_stat)) & HS_MDM_B;     /* Hostess S mdm inverted */
      outp(chan_a_cmd,RR0);                         /* index RR0 */
      SCCModemA = inp(chan_a_cmd) & SCC_MDM;        /* SCC port a modem stat */
      outp(chan_b_cmd,RR0);                         /* index RR0 */
      SCCModemB = inp(chan_b_cmd) & SCC_MDM;        /* SCC port b modem stat */
      if(!( (SCCModemA == DCD_) && (HSModemA == DSR1) )
             || SCCModemB || HSModemB) {
         printf("...................FAILED!!!");
         failed = TRUE;                             /* only port a DSR & DCD */
         break;
      }
      /*********************************************/
      /* check RTS port b to CTS and RI on port a  */
      /*********************************************/
      outp(chan_a_cmd,WR5);                         /* index WR5 */
      outp(chan_a_cmd,TX_8BITS_CHAR);               /* RTS low */
      outp(config_reg, DTR_A_OFF);                  /* port a DTR low */
      outp(chan_b_cmd,WR5);                         /* index WR5 */
      outp(chan_b_cmd,TX_8BITS_CHAR | RTS);         /* DTR low, RTS high */
      wait(4L);                                     /* 4 ms for propagation */
      HSModemA = ~(inp(modem_stat)) & HS_MDM_A;     /* Hostess S mdm inverted */
      HSModemB = ~(inp(modem_stat)) & HS_MDM_B;     /* Hostess S mdm inverted */
      outp(chan_a_cmd,RR0);                         /* index RR0 */
      SCCModemA = inp(chan_a_cmd) & SCC_MDM;        /* SCC port a modem stat */
      outp(chan_b_cmd,RR0);                         /* index RR0 */
      SCCModemB = inp(chan_b_cmd) & SCC_MDM;        /* SCC port b modem stat */
      if(!( (SCCModemA == CTS_) && (HSModemA == RI1) )
             || SCCModemB || HSModemB) {
         printf("...................FAILED!!!");
         failed = TRUE;                             /* only port a CTS & RI */
         break;
      }
   }
   /***********************************/
   /* clear both ports' DTR and RTS   */
   /***********************************/
   outp(chan_a_cmd,WR5);                         /* index WR5 */
   outp(chan_a_cmd,TX_8BITS_CHAR);               /* RTS low */
   outp(config_reg, DTR_A_OFF);                  /* port a DTR low */
   outp(chan_b_cmd,WR5);                         /* index WR5 */
   outp(chan_b_cmd,TX_8BITS_CHAR);               /* DTR low, RTS low */

   if(!failed){
      printf("...................PASSED");
   }
}

/******************************************************************************/
/* name: ModemDB9()                                                           */
/* entry: none                                                                */
/* function: modem control signal test for port 2 through DB9 loopback plug   */
/******************************************************************************/
void ModemDB9(void)
{
   int i;                         /* counter */
   int failed = FALSE;            /* true if test fails */
   int SCCModem;                  /* RR0 modem status read */
   int HSModem;                   /* Hostess S modem_stat register read */

   printf("\nPort b modem control test ");

   outp(config_reg, DTR_A_OFF);   /* clear Hostess S config register */
   SCC_Init(chan_b_cmd);          /* initialize SCC */
   SCC_Enable(chan_b_cmd);        /* enable SCC */

   for(i=0; i<=10; i++)           /* test all combinations 10 times */
   {
      /****************************/
      /* clear both DTR and RTS   */
      /****************************/
      outp(chan_b_cmd,WR5);                         /* index WR5 */
      outp(chan_b_cmd,TX_8BITS_CHAR);               /* DTR low, RTS low */
      wait(4L);                                     /* 4 ms for propagation */
      HSModem = ~(inp(modem_stat)) & HS_MDM_B;      /* Hostess S mdm inverted */
      outp(chan_b_cmd,RR0);                         /* index RR0 */
      SCCModem = inp(chan_b_cmd) & SCC_MDM;         /* get SCC modem stat */
      if(SCCModem || HSModem) {
         printf(".........................FAILED!!!");
         failed = TRUE;
         break;
      }
      /****************************/
      /* check DTR to DSR and DCD */
      /****************************/
      outp(chan_b_cmd,WR5);                         /* index WR5 */
      outp(chan_b_cmd,TX_8BITS_CHAR | DTR);         /* DTR high, RTS low */
      wait(4L);                                     /* 4 ms for propagation */
      HSModem = ~(inp(modem_stat)) & HS_MDM_B;      /* Hostess S mdm inverted */
      outp(chan_b_cmd,RR0);                         /* index RR0 */
      SCCModem = inp(chan_b_cmd) & SCC_MDM;         /* get SCC modem stat */
      if(!( (SCCModem == DCD_) && (HSModem == DSR2) )) {
         printf(".........................FAILED!!!");
         failed = TRUE;
         break;
      }
      /****************************/
      /* check RTS to CTS and RI  */
      /****************************/
      outp(chan_b_cmd,WR5);                         /* index WR5 */
      outp(chan_b_cmd,TX_8BITS_CHAR | RTS);         /* DTR low, RTS high */
      wait(4L);                                     /* 4 ms for propagation */
      HSModem = ~(inp(modem_stat)) & HS_MDM_B;      /* Hostess S mdm inverted */
      outp(chan_b_cmd,RR0);                         /* index RR0 */
      SCCModem = inp(chan_b_cmd) & SCC_MDM;         /* get SCC modem stat */
      if(!( (SCCModem == CTS_) && (HSModem == RI2) )) {
         printf("..........................FAILED!!!");
         failed = TRUE;
         break;
      }
      /****************************/
      /* check both DTR and RTS   */
      /****************************/
      outp(chan_b_cmd,WR5);                         /* index WR5 */
      outp(chan_b_cmd,TX_8BITS_CHAR | RTS | DTR);   /* DTR high, RTS high */
      wait(4L);                                     /* 4 ms for propagation */
      HSModem = ~(inp(modem_stat)) & HS_MDM_B;      /* Hostess S mdm inverted */
      outp(chan_b_cmd,RR0);                         /* index RR0 */
      SCCModem = inp(chan_b_cmd) & SCC_MDM;         /* get SCC modem stat */
      if(!( (SCCModem == SCC_MDM) && (HSModem == HS_MDM_B) )) {
         printf(".........................FAILED!!!");
         failed = TRUE;
         break;
      }
   }
   /****************************/
   /* clear both DTR and RTS   */
   /****************************/
   outp(chan_b_cmd,WR5);                         /* index WR5 */
   outp(chan_b_cmd,TX_8BITS_CHAR);               /* DTR low, RTS low */

   if(!failed){
      printf(".........................PASSED");
   }
}

/******************************************************************************/
/* name: Set_Reg_Addr()                                                       */
/* entry: uses global variable Base to calculate addresses.                   */
/* function: set the addresses for the HOSTESS S registers.                   */
/******************************************************************************/
void Set_Reg_Addr(void)
{
   chan_a_data = Base + 3;
   chan_a_cmd = Base + 1;   /* set scc control register base */
   chan_b_data = Base + 2;
   chan_b_cmd = Base;
   config_reg = Base + 4;   /* write for HOSTESS S config register */
   modem_stat = Base + 4;   /* read for modem status register */
 }


/******************************************************************************/
/* name: Test_1_2()                                                           */
/* entry: none                                                                */
/* function: test controller when ports 1 and 2 have been looped together.    */
/******************************************************************************/
int Test_1_2(void)
{

   /******************************************************/
   /* DMA RECEIVE on port a, polled transmit from port b */
   /******************************************************/
   printf("\nPolled transmit port b, DMA receive port a ");
   if(!(TxPoll_RxDMA(chan_b_cmd, chan_a_cmd)))
      printf("........FAILED!!!");
   else
      printf("........PASSED");

   /******************************************************/
   /* DMA RECEIVE on port b, polled transmit from port a */
   /******************************************************/
   printf("\nPolled transmit port a, DMA receive port b ");
   if(!(TxPoll_RxDMA(chan_a_cmd, chan_b_cmd)))
      printf("........FAILED!!!");
   else
      printf("........PASSED");

   /*****************************************************/
   /* Synchronous test Rx and Tx between port a and b   */
   /*****************************************************/
   printf("\nSynchronous Tx port a, Rx port b using TRxC");

   /* DTE source port a, DCE TRxC source (pin 15) port b */
   outp(config_reg, DTR_A_OFF | DTE_SRC_A);  /* cfg reg for Tx clock src */
   Init_Bisync(chan_a_cmd,DTE_SOURCE);       /* port a DTE source bisync */
   Init_Bisync(chan_b_cmd,DCE_SRC_TRX);      /* port b DCE TRxC source bisync */

   if(!Tx_Rx_Poll(chan_a_cmd, chan_b_cmd))
      printf("........FAILED!!!");
   else
      printf("........PASSED");

   printf("\nSynchronous Tx port a, Rx port b using RTxC");

   /* DTE source port a, DCE TRxC source (pin 15) port b */
   outp(config_reg, DTR_A_OFF | DTE_SRC_A);  /* cfg reg for Tx clock src */
   Init_Bisync(chan_a_cmd,DTE_SOURCE);       /* port a DTE source bisync */
   Init_Bisync(chan_b_cmd,DCE_SRC_RTX);      /* port b DCE RTxC source bisync */

   if(!Tx_Rx_Poll(chan_a_cmd, chan_b_cmd))
      printf("........FAILED!!!");
   else
      printf("........PASSED");

   printf("\nSynchronous Tx port b, Rx port a using TRxC");
   /* DTE source port b, DCE TRxC source (pin 15) port a */
   outp(config_reg, DTR_A_OFF | DTE_SRC_B);  /* cfg reg for Tx clock src */
   Init_Bisync(chan_a_cmd,DCE_SRC_TRX);      /* port a DCE TRxC source bisync */
   Init_Bisync(chan_b_cmd,DTE_SOURCE);       /* port b DTE source bisync */
   if(!Tx_Rx_Poll(chan_b_cmd, chan_a_cmd))
      printf("........FAILED!!!");
   else
      printf("........PASSED");

   printf("\nSynchronous Tx port b, Rx port a using RTxC");
   /* DTE source port b, DCE TRxC source (pin 15) port a */
   outp(config_reg, DTR_A_OFF | DTE_SRC_B);  /* cfg reg for Tx clock src */
   Init_Bisync(chan_a_cmd,DCE_SRC_RTX);      /* port a DCE RTxC source bisync */
   Init_Bisync(chan_b_cmd,DTE_SOURCE);       /* port b DTE source bisync */
   if(!Tx_Rx_Poll(chan_b_cmd, chan_a_cmd))
      printf("........FAILED!!!");
   else
      printf("........PASSED");
}

/******************************************************************************/
/* name: Tx_Rx_Poll()                                                         */
/* entry: TxChanCmd is command reg for channel that does Tx                   */
/*        RxChanCmd is command reg for channel that does Rx                   */
/* function: test controller when ports have been looped together by          */
/*           polled Tx and polled Rx.                                         */
/******************************************************************************/
int Tx_Rx_Poll(int TxChanCmd, int RxChanCmd)
{
   int TimedOut;
   int i,j;
   int byte_count;

   byte_count = Data_Init();

   for(i = 0; i <= byte_count; i++)        /* output chars using polled mode */
   {
      TimedOut = TRUE;                           /* assume we will time out */
      for(j=0; j<=100; j++)
      {
         wait(1L);                               /* wait 1 ms, 101 ms max */
         outp(TxChanCmd, RR0);                   /* index RR0 */
         if(inp(TxChanCmd) & 0x4)                /* check Tx buff empty */
         {
            outp(TxChanCmd + 2, sys_data_tx[i]); /* write data to Tx port */
            TimedOut = FALSE;                    /* didn't time out */
            break;
         }
      }
      if(TimedOut)
         break;

      TimedOut = TRUE;                           /* assume we will time out */
      for(j=0; j<=100; j++)
      {
         wait(1L);                               /* wait 1 ms, 101 ms max */
         outp(RxChanCmd, RR0);                   /* index RR0 */
         if(inp(RxChanCmd) & 0x1)                /* check Rx char available */
         {
            sys_data_rx[i] = inp(RxChanCmd + 2); /* read data from Rx port */
            TimedOut = FALSE;                    /* didn't time out */
            break;
         }
      }
      if(TimedOut)
         break;
   }

   if(TimedOut)
      return(FALSE);                              /* failed, loop timed out */

   for(i = 0; i <= byte_count; i++)               /* verify data in memory */
      if(sys_data_rx[i] != sys_data_tx[i])
         return(FALSE);

   return(TRUE);
}


/******************************************************************************/
/* name: TxPoll_RxDMA()                                                       */
/* entry: TxChanCmd is command reg for channel that does Tx                   */
/*        RxChanCmd is command reg for channel that does Rx                   */
/* function: test controller when ports have been looped together.            */
/*           uses DMA for receiving chars and polling to transmit chars.      */
/******************************************************************************/
int TxPoll_RxDMA(int TxChanCmd, int RxChanCmd)
{
   int i,j;                     /* counters */
   int byte_count;              /* byte count holder */
   int pass = TRUE;
   int TimedOut;                /* true if operation timed out */

   byte_count = Data_Init();    /* load chars into memory for Tx */

   outp(config_reg, DTR_A_OFF); /* HOSTESS S config reg options off */

   /* Tx port SCC initialization */
   SCC_Init(TxChanCmd);    /* initialize SCC communication parms for Tx port */
   SCC_Enable(TxChanCmd);  /* enable polled mode for Tx port */

   /* Rx port SCC initialization for DMA w/req */
   SCC_Init(RxChanCmd);                /* initialize com params for Rx port */
   SCC_DMA_Init(RxChanCmd, WREQ_RX);   /* enable dma w/req on rcv for Rx port */

   if(RxChanCmd == chan_a_cmd)             /* init chan a for Rx? */
   {
      outp(config_reg, DMA1_W);            /* W/REQ on port 1, dma chan 1 */
      Sys_DMA_Init( 1, WRITE, sys_rx_abs, byte_count);
      outp(WR_SNGL_MSK, 0x1);              /* unmask dma chan 1 (start DMA) */
   }
   if(RxChanCmd == chan_b_cmd)             /* init chan b for Rx? */
   {
      outp(config_reg, DMA3_B_W);          /* W/REQ on port 2, dma chan 3 */
      Sys_DMA_Init( 3, WRITE, sys_rx_abs, byte_count);
      outp(WR_SNGL_MSK, 0x3);              /* unmask dma chan 3 (start DMA) */
   }

   for(i = 0; i <= byte_count; i++)        /* output chars using polled mode */
   {
      TimedOut = TRUE;                           /* assume we will time out */
      for(j=0; j<=100; j++)
      {
         wait(1L);                               /* wait 1 ms, 101 ms max */
         outp(TxChanCmd, RR0);                   /* index RR0 */
         if(inp(TxChanCmd) & 0x4)                /* check Tx buff empty */
         {
            outp(TxChanCmd + 2, sys_data_tx[i]); /* write data to Tx port */
            TimedOut = FALSE;                    /* didn't time out */
            break;
         }
      }
      if(TimedOut)
         break;
   }

   if(TimedOut)
      return(FALSE);          /* failed, loop timed out */

   wait(500L);    /* wait 500 ms for last 3 or 4 chars to be Tx'd and Rx'd */

   if(RxChanCmd == chan_a_cmd)              /* stop DMA on chan a ? */
      outp(WR_SNGL_MSK, CH1_OFF); /* mask off the DMA1 req line (stops DMA) */

   if(RxChanCmd == chan_b_cmd)              /* stop DMA on chan b ? */
      outp(WR_SNGL_MSK, CH3_OFF); /* mask off the DMA3 req line (stops DMA) */

   outp(CLR_PNTR, 0x0);                     /* clear byte pointer F/F */

   /* dma rcv xfers should have occurred during the above polled xmits */
   for(i = 0; i <= byte_count; i++)         /* verify data in memory from DMA */
      if(sys_data_rx[i] != sys_data_tx[i])
         pass = FALSE;

   return(pass);           /* true if routine passed, false if failed */
}

/******************************************************************************/
/* name: Init_Bisync()                                                        */
/* entry: command register address for scc of interest.                       */
/* function: initialize scc for bisync operation in polled mode               */
/******************************************************************************/
void Init_Bisync(int Chan_Cmd_Reg, int ClockMode)
{
   int temp;

   /* reset the desired channel (channel reset is common) */
   outp(Chan_Cmd_Reg, WR9);                    /* set index reg for wr9 */
   if(Chan_Cmd_Reg == chan_a_cmd)              /* init chan a ? */
      outp(Chan_Cmd_Reg, RESET_CHA);           /* reset channel a */
   if(Chan_Cmd_Reg == chan_b_cmd)              /* init chan b ? */
      outp(Chan_Cmd_Reg, RESET_CHB);           /* reset channel b */

   brtc_lsb = (char)(X1_baud[BaudIndex] & 0xFF);
   outp(Chan_Cmd_Reg, WR12);                   /* set index reg for wr12*/
   outp(Chan_Cmd_Reg, brtc_lsb);               /* baud rate const LSB */
   temp = (X1_baud[BaudIndex] >> 8) & 0xFF;    /* calc MSB of brtc */
   brtc_msb = (char)temp;                      /* note: using x1 clock mode */
   outp(Chan_Cmd_Reg, WR13);                   /* set index reg for wr13*/
   outp(Chan_Cmd_Reg, brtc_msb);               /* baud rate const MSB */

   outp(Chan_Cmd_Reg,WR4);                     /* index WR4 */
   outp(Chan_Cmd_Reg,X1_CLOCK|_16BIT_SYNC);    /* 1X clk, 16 bit sync char */

   outp(Chan_Cmd_Reg,WR10);                    /* index WR10 */
   outp(Chan_Cmd_Reg,NRZ);                     /* NRZ mode (all zeros) */

   outp(Chan_Cmd_Reg,WR6);                     /* index WR6 */
   outp(Chan_Cmd_Reg,0x16);                    /* sync char1 (arbitrary) */

   outp(Chan_Cmd_Reg,WR7);                     /* index WR7 */
   outp(Chan_Cmd_Reg,0x16);                    /* sync char2 (arbitrary) */

   outp(Chan_Cmd_Reg,WR3);
   outp(Chan_Cmd_Reg,RX_8BITS_CHAR | LOAD_INHIBIT); /* sync char load inhibit */

   outp(Chan_Cmd_Reg,WR5);
   outp(Chan_Cmd_Reg,TX_8BITS_CHAR);           /* 8 bit Tx chars */

   switch( ClockMode )
   {
      case DCE_SRC_RTX:
         outp(Chan_Cmd_Reg,WR11);
         outp(Chan_Cmd_Reg,TC_EQ_RTxC);        /* Tx clock = RTxC pin on SCC */
         break;
      case DCE_SRC_TRX:
         outp(Chan_Cmd_Reg,WR11);
         outp(Chan_Cmd_Reg,TC_EQ_TRxC);        /* Tx clock = RTxC pin on SCC */
         break;
      case DTE_SOURCE:
         outp(Chan_Cmd_Reg,WR11);
         outp(Chan_Cmd_Reg,TX_CLK_EQ_BRG);    /* Tx clock = brg output */
         break;
      default:
         break;
   }

   outp(Chan_Cmd_Reg,WR14);
   outp(Chan_Cmd_Reg,BRG_EQ_SYS_CLK | BRG_ENABLE);  /* BRG source = sysclock */

   outp(Chan_Cmd_Reg,WR3);
   outp(Chan_Cmd_Reg,RX_8BITS_CHAR | LOAD_INHIBIT | RX_ENABLE); /* enable Rx */

   outp(Chan_Cmd_Reg,WR5);
   outp(Chan_Cmd_Reg,TX_8BITS_CHAR | TX_ENABLE); /* 8 bit char, Tx enable */
}

/******************************************************************************/
/* name: Test_IRQ()                                                           */
/* entry: Uses global IRQ                                                     */
/* function: test port 2 with a loop back plug installed in the DB9 connector.*/
/*           Interrupt driven I/O on port 2 DB9                               */
/******************************************************************************/
void Test_IRQ(void)
{
   int Rx_Count, byte_count, i;        /* byte counters for Tx and Rx */
   int INT_Type;                       /* vector associated with IRQ */
   int TimedOut = FALSE;               /* timeout for Rx loop */

   printf("\nIRQ test via port b DB9 connector ");

   byte_count = Data_Init(); /* load Tx chars into memory, clear Rx buffer */

   outp(config_reg, 0x0);    /* set HOSTESS S config reg to 0 for insurance */
   SCC_Init(chan_b_cmd);     /* initialize SCC communication parms for port 2 */

   outp(chan_b_cmd, RR0);                  /* index RR0 */
   if((inp(chan_b_cmd) & 0x5) != 0x4)      /* should be 4 after SCC_Init() */
   {
      printf(".................FAILED!!!");
      return;                              /* bail out */
   }

   SCC_Enable(chan_b_cmd);              /* enable Tx, Rx and brg for port 2 */

   /* set interrupt type for SCC */
   outp(chan_b_cmd, WR15);              /* set index reg for wr15 */
   outp(chan_b_cmd, 0x0);               /* don't allow special Rx cond to int */

   outp(chan_b_cmd, WR1);               /* set index reg for wr1 */
   outp(chan_b_cmd, TX_INT_ENABLE);     /* set SCC for Tx interrupt */
   outp(chan_b_cmd, WR9);               /* set index reg for wr9 */
   outp(chan_b_cmd, MIE);               /* enable SCC interrupts */

   Interrupted = FALSE;                 /* indicate no ints yet */
   Intrpt_Cnt = 0;                      /* init the Tx char counter */
   Rx_Count = 0;                        /* init the Rx char counter */
   Char_Cnt = byte_count;               /* let ISR know how many chars to TX */

   en_irq( IRQ );                       /* enable interrupt and init vector */

   outp(chan_b_data, sys_data_tx[0]);   /* output data, start interrupts */

   while((Rx_Count < byte_count) && (!TimedOut))
   {                                    /* start polling for Rx chars */
      TimedOut = TRUE;                  /* assume we will time out */
      for(i=0; i<=100; i++)
      {
         wait(1L);                      /* wait 1 ms, 101 ms max in for loop */
         outp(chan_b_cmd, RR0);
         while((inp(chan_b_cmd) & 0x1) == 0x1)  /* check RX char available */
         {
            sys_data_rx[Rx_Count++] = inp(chan_b_data); /* get data from scc */
            outp(chan_b_cmd, RR0);                      /* keep checking RR0 */
            TimedOut = FALSE;                           /* didn't timeout */
         }
         if(!TimedOut)
            break;
      }
   }
   outp(chan_b_cmd, WR9);               /* set index reg for wr9 */
   outp(chan_b_cmd, RESET_CHB);         /* reset channel b */

   dis_irq( IRQ );                      /* disable the irq and restore vector */

   if(TimedOut)
   {
      printf(".................FAILED!!!");
      return;                           /* bail out */
   }

   /* when we get here, all chars have been Tx'd and Rx'd */
   for(i = 0; i <= byte_count; i++)     /* verify Rx data from ISR */
   {
      if(sys_data_rx[i] != sys_data_tx[i])
      {
         fail = TRUE;
         break;
      }
   }
   if(fail)
/*    printf("miscompare byte = %x ....FAILED!!!", i);  */
      printf(".................FAILED!!!");
   else
      printf(".................PASSED");
}

/******************************************************************************/
/* name: hndirq()                                                             */
/* entry: this is the ISR that en_irq() sets the interrupt vector for         */
/* function: transmits characters from port 2                                 */
/******************************************************************************/
void hndirq(void)
{
   Interrupted++;     /* global that indicates the number of interrupts */

   outp(chan_b_cmd, RR0);
   while( (inp(chan_b_cmd) & 0x4 ) && (Intrpt_Cnt <= Char_Cnt) )
   {
      outp(chan_b_data, sys_data_tx[++Intrpt_Cnt]);  /* output data to scc */
      outp(chan_b_cmd, RR0);
   }
}

/*****************************************************************************/
/* Name:  calibrate_wait                                                     */
/* Purpose: Compute the time factor for the wait() function.                 */
/* Entry: Nothing.                                                           */
/* Exit: Global variable time_factor is modified.                            */
/*****************************************************************************/

void calibrate_wait(void)
{
   int retrys;                      /* retry counter if elapsed time invalid */
   long int i;                      /* loop counter */
   long int elapsed_time;           /* time elapsed during looping */
   long int start_time;             /* starting time */
   long int end_time;               /* ending time */
   long int nloops;                 /* number of loops to execute */

   retrys = 0;
   elapsed_time = 0;
   while(elapsed_time < 1)           /* while elapsed time invalid */
   {                                 /*  first try might span clock rollover */

      if(retrys == 3)                /* if computer doesn't support clk ticks*/
      {
         time_factor = 400;          /*   estimate it */
         return;
      }

      nloops = 200000L;
      start_time = *p_clktick;
      for(i = 0;i < nloops;i++);       /* loop being timed */
      end_time = *p_clktick;
      elapsed_time = end_time - start_time;
      retrys++;
   }

   elapsed_time *= 54;                 /* convert ticks to milliseconds */
   time_factor = i/elapsed_time;       /* convert to loops per ms */
   if(time_factor < 1)                 /* a very slow computer */
      time_factor = 1;
}

/*****************************************************************************/
/* Name:  wait                                                               */
/* Purpose: Do nothing for the length of time specified.  Accurate to within */
/*          about 1ms, the faster the machine, the better because function   */
/*          call overhead is reduced.                                        */
/* Entry: wtime - Wait time in milliseconds.                                 */
/*        time_factor - Global variable, is number of for loops required     */
/*                      to equal 1 millisecond.                              */
/* Exit: Nothing.                                                            */
/* Note: Max wtime is computer dependant, the faster the computer the larger */
/*       time_factor will be.  On a fast 33MHz machine estimate won't get    */
/*       overflow if wtime < 3 million milliseconds (conservatively).        */
/*****************************************************************************/

void wait(long int wtime)
{
   long int i;

   wtime *= time_factor;               /* number of loops in 1 millisecond */
   for(i = 0;i < wtime;i++);
}

/**************************/
/* include menu functions */
/**************************/
#include "menus.c"


