/***************************************************************************/
/* FILE: HS.H                                                              */
/* Copyright (C) 1992 Comtrol Corporation                                  */
/* Global definitions for HS.C                                             */
/***************************************************************************/

/***************************************************************************/
/* Misc. definitions                                                       */
/***************************************************************************/

#define TRUE 1
#define FALSE 0

#define DTRQ_TX     0x4    /* for SCC_DMA_Init() param */
#define WREQ_TX     0x3    /* for SCC_DMA_Init() param */
#define WREQ_RX     0x1    /* for SCC_DMA_Init() param */

#define WRITE       1      /* for Sys_DMA_Init() param */
#define READ        2      /* for Sys_DMA_Init() param */

#define DTE_SOURCE  1      /* for InitBysinc() param */
#define DCE_SRC_RTX 2      /* for InitBysinc() param */
#define DCE_SRC_TRX 3      /* for InitBysinc() param */

/***************************************************************************/
/* SCC register defines (8530 serial communications controller)            */
/***************************************************************************/

/* write register index values for 8530 SCC */
#define WR0      0x0
#define WR1      0x1
#define WR2      0x2
#define WR3      0x3
#define WR4      0x4
#define WR5      0x5
#define WR6      0x6
#define WR7      0x7
#define WR8      0x8
#define WR9      0x9
#define WR10     0xA
#define WR11     0xB
#define WR12     0xC
#define WR13     0xD
#define WR14     0xE
#define WR15     0xF

/* read register index values for 8530 SCC */
#define RR0      0x0
#define RR1      0x1
#define RR2      0x2
#define RR3      0x3
#define RR6      0x6
#define RR7      0x7
#define RR8      0x8
#define RR10     0xA
#define RR12     0xC
#define RR13     0xD
#define RR15     0xF

/***************************************************************************/
/* baud rate generator time constants - x16 clock mode                     */
/* the baud rates below are divisors based on 8.192 Mhz clock              */
/***************************************************************************/
#define BPS19200 0xB       /* %error +2.56% */
#define BPS9600  0x19      /* %error -1.23% */
#define BPS4800  0x33      /* %error +0.63% */
#define BPS2400  0x69      /* %error -0.31% */
#define BPS1200  0xD3      /* %error +0.16% */
#define BPS600   0x1A9     /* %error -0.08% */
#define BPS300   0x353     /* %error +0.04% */
#define BPS150   0x6A9     /* %error -0.02% */

/***************************************************************************/
/* baud rate generator time constants - x1 clock mode                      */
/***************************************************************************/
#define X1BPS19200 0xD3
#define X1BPS9600  0x1A9
#define X1BPS4800  0x353
#define X1BPS2400  0x6A9
#define X1BPS1200  0xD55
#define X1BPS600   0x1AA6
#define X1BPS300   0x3555
#define X1BPS150   0x6AAB

/***************************************************************************/
/* SCC Write Register Definitions                                          */
/***************************************************************************/

/* Write Register 0 - command register */
#define RESET_EXT        0x10   /* reset external status interrupt */
#define RESET_TX_INT     0x28   /* wait for another char to be loaded b-4 int */
#define ERROR_RESET      0x30   /* reset error from rx */
#define RESET_IUS        0x38   /* allow interrupts from lower priority SCC's */

/* Write Register 1 - interrupt and DMA definition */
#define EXT_INT_ENABLE   0x01   /* External status interrupt enable */
#define TX_INT_ENABLE    0x02   /* Tx interrupt enable */
#define PARITY_SPECIAL   0x04   /* Special receive condition enable */
#define RX_INT_ENABLE    0x10   /* Rx interrupt enable */
#define TXDMA_W          0xC0   /* enable W/REQ DMA request on Tx */
#define RXDMA_W          0xe0   /* enable W/REQ DMA request on Rx */

/* Write Register 2 - interrupt vector (vector modified mode offset) */

/* Write Register 3 - Rx parameters and controls */
#define RX_ENABLE        0x01      /* enable Rx */
#define LOAD_INHIBIT     0x02      /* sync character load inhibit */
#define RX_5BITS_CHAR    0x00      /* 5 bits per char Rx (bits 6 & 7 clear) */
#define RX_6BITS_CHAR    0x80      /* 6 bits per char Rx */
#define RX_7BITS_CHAR    0x40      /* 7 bits per char Rx */
#define RX_8BITS_CHAR    0xC0      /* 8 bits per char Rx */

/* Write Register 4 - Tx/Rx parameters */
#define PARITY_ENABLE    0x01
#define PARITY_EVEN      0x02
#define PARITY_ODD       0x00   /* bit 1 clear */
#define SYNC_MODE_ENAB   0x00   /* sync modes enable (bits 2 & 3 clear) */
#define ONE_STOP_BIT     0x04   /* one stop bit */
#define ONE$_STOP_BITS   0x08   /* one and 1/2 stop bits */
#define TWO_STOP_BITS    0x0c   /* two stop bits */
#define X1_CLOCK         0x00   /* 1 x clock (bits 6 & 7 clear) */
#define X16_CLOCK        0x40   /* 16 x clock */
#define X32_CLOCK        0x80   /* 32 x clock */
#define X64_CLOCK        0xC0   /* 64 x clock */
#define _8BIT_SYNC       0x00   /* 8 bit sync char (monosync) (bits 4&5 clr) */
#define _16BIT_SYNC      0x10   /* 16 bit sync character (bisync) */
#define SDLC_MODE        0x20   /* sdlc mode */
#define EXT_SYNC         0x30   /* external sync mode */
#define RXTX_PARMS       0x44   /* times 16 clock, 1 stop, no parity, no sync */

/* Write Register 5 - Tx paramerameters and controls */
#define RTS              0x02
#define TX_ENABLE        0x08
#define BREAK            0x10
#define TX_5BITS_CHAR    0x00      /* 5 bits per char Tx */
#define TX_6BITS_CHAR    0x40      /* 6 bits per char Tx */
#define TX_7BITS_CHAR    0x20      /* 7 bits per char Tx */
#define TX_8BITS_CHAR    0x60      /* 8 bits per char Tx */
#define DTR              0x80

/* Write Register 6 - sync character or SDLC address field */

/* Write Register 7 - sync character or SDLC flag */

/* Write Register 8 - transmit buffer */

/* Write Register 9 - master interrupt control */
#define VIS              0x01
#define NV               0x02
#define DLC              0x04
#define MIE              0x08     /* Master Interrupt Enable */
#define STATUS_LO        0x00
#define STATUS_HI        0x10
#define RESET_CHB        0x40     /* reset channel b */
#define RESET_CHA        0x80     /* reset channel a */
#define HARDWARE_RESET   0xc0


/* Write Register 10 - miscellaneous Tx/Rx control bits */
#define NRZ              0x00     /* NRZ mode (bits 5 & 6 clear) */

/* Write Register 11 - clock mode control */
#define TX_CLK_EQ_BRG    0x15     /* transmit clock is brg output on Trxc pin */
#define CLK_SOURCE       0x50     /* rx and tx clock = baud rate generator */
#define TC_EQ_RTxC       0x0      /* Rx clock = RTxC pin on SCC */
#define TC_EQ_TRxC       0x20     /* Rx clock = TRxC pin on SCC */

/* Write Register 12 - lower byte of baud generator time constant */

/* Write Register 13 - upper byte of baud generator time constant */

/* Write Register 14 - DMA and BRG control bits */
#define BRG_ENABLE       0x01
#define BRG_EQ_SYS_CLK   0x02  /* baud rate generator source = sysclock */
#define TXDMA_DTR        0x4   /* enable DTR/REQ DMA req on xmit */

/* Write Register 15 - external/STATUS interrupt control */
#define DCD_IE           0x08
#define CTS_IE           0x20
#define BREAK_IE         0x80

/***************************************************************************/
/* SCC Read Register Definitions                                           */
/***************************************************************************/

/* Read Register 0 - Tx/Rx BUFFER STATUS and external STATUS */
#define RX_BUFFER        0x01
#define TX_BUFFER        0x04
#define DCD_             0x08
#define CTS_             0x20
#define BREAK_COND       0x80
#define SCC_MDM          0x28  /* DCD_ | CTS_  to isolate modem bits */

/* Read Register 1 - special receive condition STATUS */
#define ALLSENT          0x01
#define PARITY_ERROR     0x10
#define OVERRUN_ERROR    0x20
#define FRAMING_ERROR    0x40

/* Read Register 2 - interrupt vector (modified mode) */

/* Read Register 3 - interrupt pending STATUS */

/* Read Register 8 - receive data register */

/* Read Register 10 - miscellaneous STATUS bits */

/* Read Register 12 - value stored in WR12 */

/* Read Register 13 - value stored in WR13 */

/* Read Register 15 - value stored in WR15 */


/***************************************************************************/
/* HOSTESS S configuration register defines                                */
/***************************************************************************/

#define DMA1_DTR   0x2    /* HOSTESS S command reg value for DTR/REQ on chn a */
                          /* sets DMA sel 0 to activate DRQ1 */
#define DMA1_W     0x8    /* HOSTESS S command reg value for W/REQ on chn a */
                          /* sets DMA sel 1 to activate DRQ1 */
#define DMA3_DTR   0x3    /* HOSTESS S command reg value for DTR/REQ on chn a */
                          /* sets DMA sel 0 to activate DRQ3 */
#define DMA3_W     0xc    /* HOSTESS S command reg value for W/REQ on chn a */
                          /* sets DMA sel 1 to activate DRQ3 */
#define DMA3_B_W   0x10   /* HOSTESS S command reg value for W/REQ on chn b */
                          /* sets DMA sel 2 to activate DRQ3 */
#define DTR_A_OFF  0x20   /* command reg value to unassert DTR on chn a */

#define DTE_SRC_A  0x40   /* Tx clock DTE source port a */

#define DTE_SRC_B  0x80   /* Tx clock DTE source port b */




/***************************************************************************/
/* HOSTESS S modem stat register defines                                   */
/***************************************************************************/

#define DSR1      0x1    /* port 1 data set ready bit */
#define RI1       0x2    /* port 1 ring indicator bit */
#define DSR2      0x4    /* port 2 data set ready bit */
#define RI2       0x8    /* port 2 ring indicator bit */
#define HS_MDM_A  0x3    /* RI1 | DSR1  used to isolate modem bits for port 1 */
#define HS_MDM_B  0xC    /* RI2 | DSR2  used to isolate modem bits for port 2 */

/***************************************************************************/
/* System DMA controller register defines (resides on PC motherboard)      */
/***************************************************************************/
#define DMA1_PG_REG 0x83
#define DMA3_PG_REG 0x82
#define CH1_ADD     0x2
#define CH1_BC      0x3
#define CH3_ADD     0x6
#define CH3_BC      0x7
#define STAT_CMD    0x8
#define WR_RQ       0x9
#define WR_SNGL_MSK 0xA
#define WR_MODE     0xB
#define CLR_PNTR    0xC
#define TEMP_MCLR   0xD
#define CLR_MASK    0xE
#define MASK_ALL    0xF
#define ADDR_INC    0x0     /* address increment ( as opposed to decrement ) */
#define CH1_SEL     0x1     /* select DMA channel 1 */
#define CH3_SEL     0x3     /* select DMA channel 3 */
#define WRITE_XFER  0x4     /* set for Write transfer, DMA from I/O to memory */
#define READ_XFER   0x8     /* set for Read transfer, DMA from memory to I/O */
#define AUTO_INIT   0x10    /* auto init (keep cycling through defined memory */
#define SINGLE_MODE 0x40    /* single byte per request */
#define CH1_OFF     0x5     /* DMA channel 1 mask, turns dma off */
#define CH3_OFF     0x7     /* DMA channel 3 mask, turns dma off */

/***************************************************************************/
/* Menu definitions.                                                       */
/***************************************************************************/

/* enum for main menu */
enum {AUTO, SETBASE, SETIRQ, SETBAUD, R1_POLLED, X1_POLLED, X1_DMA1,
      X1_DMA3, R2_POLLED, X2_POLLED, X2_DMA3, QUIT};

/* enum for DMA selection, used for port 1 */
enum {W_REQ, DTR_REQ};

/* Baud rate generator time constants array */
int baud[] =
{BPS19200, BPS9600, BPS4800, BPS2400, BPS1200, BPS600, BPS300, BPS150};

/* Baud rate generator time constants array for X1 clock used for sync */
int X1_baud[] =   {
   X1BPS19200, X1BPS9600, X1BPS4800, X1BPS2400,
   X1BPS1200,  X1BPS600,  X1BPS300,  X1BPS150   };

/* Human readable baud rates */
int act_baud[] =
{19200, 9600, 4800, 2400, 1200, 600, 300, 150};

/***************************************************************************/
/* global (static) variable definitions                                    */
/***************************************************************************/

static char sys_data_tx[3000]; /* global data buffer for transmit data */
static char sys_data_rx[3000]; /* global data buffer for receive data */

unsigned long sys_rx_abs;
unsigned long sys_tx_abs;
int MainChoice, IRQ, Base, BaudIndex;   /* menu choices */
int DmaTypeCur;
unsigned chan_a_data;          /* I/O base + 3   */
unsigned chan_a_cmd;           /* I/O base + 1   */
unsigned chan_b_data;          /* I/O base + 2   */
unsigned chan_b_cmd;           /* I/O base + 0   */
unsigned config_reg;           /* Hostess Sync config register for dma, clck, dtr */
                          /* base io + 4 */
int modem_stat;           /* modem read only status register */
                          /* I/O base + 4 */
char brtc_lsb;            /* used to set up the baud rate time constant LSB */
char brtc_msb;            /* used to set up the baud rate time constant MSB */
int fail = FALSE;         /* to determine if a test fails */

int Interrupted;          /* indicates when an interrupt has occured */
int Intrpt_Cnt;           /* counts the number of chars at interrupt time */
int Char_Cnt;             /* counts the number of chars to be rx'd */

int good_resp;            /* for menu functions */
int Choice;               /* for menu functions */

long int time_factor;     /* for wait(), in loops per millisecond */

/* system clock tick counter occupies two bytes starting at 40:6C */
unsigned long far *p_clktick = (unsigned long far *)0x0040006C;
