         page 58,132
         TITLE    minthnd
; Name: minthnd.asm
; Copyright (C) 1992 Comtrol Corporation
; Purpose: Enable and disable interrupts, invoke "C" interrupt handler when
;          an interrupt occurs.
; Entry: Procedures en_irq and dis_irq receive 1 parameter from the calling
;        "C" function, the interrupt number.
; Exit: Nothing
; Author: Craig Harrison
; Company: COMTROL (TM) Corporation
; Revision: Original on date unknown.
; Revision: 12-31-88 by Craig Harrison to support interrupts 9 thru 15.
;           Changed program so interrupt information is stored in a table
;           instead of in "case" statements.
; Revision: 6-9-90 by Craig Harrison.  Add EOI to master and slave 8259
;           for interrupts 9 thru 15.

_TEXT    segment  byte  public  'CODE'
_TEXT    ends
_DATA    segment  word  public  'DATA'
_DATA    ends
CONST    segment  word  public  'CONST'
CONST    ends
_BSS     segment  word  public  'BSS'
_BSS     ends


; tps irqnum           equ     4             ;offset to irq number parameter
irqnum           equ     6             ;offset to irq number parameter
eoi              equ     20h           ;nonspecific End of Interrupt command
len_idat_st      equ     5             ;length of structure idat_st

idat_st          STRUC                 ;structure for interrupt information
msk              db      ?             ;mask value for 8259 mask register
off              dw      ?             ;offset into vector table
io8259           dw      ?             ;I/O address of 8259 mask register
idat_st          ENDS

DGROUP   group  CONST,_BSS,_DATA
         assume cs:_TEXT,ds:DGROUP,ss:DGROUP,es:DGROUP


_DATA    segment

pirq_offset      dw      ?             ;previous IRQ offset save
pirq_segment     dw      ?             ;previous IRQ segment save
mask_8259        db      ?             ;8259 mask register value save
io_eoi           dw      ?             ;8259 I/O address for EOI

;Table with interrupt data
;  Each entry has format <mask,vector address,8259 address>
idat     idat_st         <0fbh,28h,21h>   ;IRQ2
         idat_st         <0f7h,2ch,21h>   ;IRQ3
         idat_st         <0efh,30h,21h>   ;IRQ4
         idat_st         <0dfh,34h,21h>   ;IRQ5
         idat_st         <0bfh,38h,21h>   ;IRQ6
         idat_st         <07fh,3ch,21h>   ;IRQ7
         idat_st         <07fh,3ch,21h>   ;IRQ8 send it to IRQ3 for safety
         idat_st         <0fbh,28h,21h>   ;IRQ9 same as IRQ2
         idat_st         <0fbh,1c8h,0a1h> ;IRQ10
         idat_st         <0f7h,1cch,0a1h> ;IRQ11
         idat_st         <0efh,1d0h,0a1h> ;IRQ12
         idat_st         <0dfh,1d4h,0a1h> ;IRQ13
         idat_st         <0bfh,1d8h,0a1h> ;IRQ14
         idat_st         <07fh,1dch,0a1h> ;IRQ15

_DATA    ends


_TEXT    segment

;******************************************************************************
;Procedure name: en_irq
;Purpose: Enable interrupt.  The IRQ number received from the calling function
;   is used to select the proper value to offset into the interrupt vector
;   table (put in BX), and to select a value to be used in writing to the Mask
;   Register in the 8259 Peripheral Interrupt Controller (put in DL).  The
;   segment and offset values contained in the location pointed to by 0:BX
;   are saved in "pirq_segment" and "pirq_offset".  The location in the
;   vector table for the given IRQ is then updated with the segment and
;   offset of the proc "irq".  The 8259 Mask Register contents are saved
;   in "mask_8259" and the proper IRQ line masked ON.  No other bits in the
;   Mask Reg are changed.

;tps Entry:  [bp + 4]:  IRQ number.
;Entry:  [bp + 6]:  IRQ number.

;Exit:  Nothing
;******************************************************************************

         public  _en_irq
_en_irq  proc    far

         ;Establish stack frame and save registers
         push    bp                    ;save old frame pointer
         mov     bp,sp                 ;establish current frame pointer
         push    si                    ;save registers
         push    di
         mov     ax,0                  ;set es to bottom of memory
         mov     es,ax

         ;Get index into table idat
         xor     bx,bx                 ;clear BX
         mov     bl,[bp+irqnum]        ;get IRQ number parameter
         sub     bl,2                  ;convert IRQ number into an index
         mov     al,len_idat_st        ;  into idat
         mul     bl                    ;  and leave it
         mov     si,ax                 ;  in SI

        ;Set up vector table & 8259 interrupt mask register
         mov     bx,[idat+si].off      ;ES:BX-> IRQX interrupt vector
         mov     ax,es:[bx]            ;save previous vector offset
         mov     pirq_offset,ax
         mov     ax,es:[bx+2]          ;save previous vector segment
         mov     pirq_segment,ax

         cli                           ;disable interrupts
         mov     ax,offset irq         ;offset of proc "irq"
         mov     es:[bx],ax            ;  into IRQX vector table address
         mov     ax,cs                 ;segment of proc "irq"
         mov     es:[bx+2],ax          ;  into IRQX vector table address

         mov     dx,[idat+si].io8259   ;get 8259 mask reg I/O address in DX
         in      al,dx                 ;get current interrupt mask from 8259
         nop
         nop
         mov     mask_8259,al          ;save current interrupt mask
         and     al,[idat+si].msk      ;allow IRQX interrupts
         out     dx,al                 ;write it back to 8259
         nop
         nop
         sti                           ;enable interrupts

         dec     dx                    ;compute 8259 OCW2 address for EOI
         mov     io_eoi,dx             ;  and save it for proc irq

         ;Return
         pop     di                    ;restore registers
         pop     si
         pop     bp
         ret
_en_irq  endp


;******************************************************************************
;Procedure name: dis_irq
;Purpose: Enable interrupt. The IRQ number received from the calling function
;     is used to select the proper value to offset into the interrupt vector
;     table (put in BX).  Then the vector table and 8259 Mask Register
;     are restored to their states before "en_irq" was executed.
;Entry:  [bp + 4]:  IRQ number.
;Exit:  Nothing
;******************************************************************************

          public  _dis_irq
_dis_irq  proc    far

         ;Establish stack frame and save registers
         push    bp                    ;save old frame pointer
         mov     bp,sp                 ;establish current frame pointer
         push    si                    ;save registers
         push    di
         mov     ax,0                  ;set es to bottom of memory
         mov     es,ax

         ;Get index into table idat
         xor     bx,bx                 ;clear BX
         mov     bl,[bp+irqnum]        ;get IRQ number parameter
         sub     bl,2                  ;convert IRQ number into an index
         mov     al,len_idat_st        ;  into idat
         mul     bl                    ;  and leave it
         mov     si,ax                 ;  in SI

         ;Restore vector table & 8259 interrupt mask register
         mov     bx,[idat+si].off      ;ES:BX-> IRQX interrupt vector
         cli                           ;clear interrupts
         mov     ax,pirq_offset        ;restore previous vector offset
         mov     es:[bx],ax
         mov     ax,pirq_segment       ;restore previous vector segment
         mov     es:[bx+2],ax

         mov     dx,[idat+si].io8259   ;get 8259 mask reg I/O address in DX
         mov     al,mask_8259          ;get saved 8259 interrupt mask
         out     dx,al                 ;  and write it back to 8259
         nop
         nop
         sti                           ;enable interrupts

         ;Return
         pop     di                    ;restore registers
         pop     si
         pop     bp
         ret
_dis_irq  endp


;******************************************************************************
;Procedure name: irq
;Purpose:  Interrupt handler.  This proc is executed when IRQ? occurres.  It
;   performs a call to "C" function "hndirq".  Before the call it sets up ES,
;   DS, and SS to the values used by the "C" program.  This is necessary for
;   the case where these are not pointing to the "C" dgroup when the
;   interrupt occurs.
;Entry:  Nothing
;Exit:  Nothing
;******************************************************************************

         extrn  _hndirq:far            ;hndirq is a "C" function
irq      proc
         push ax                       ; save all registers
         push bx
         push cx
         push dx
         push bp
         push si
         push di
         push ds
         push es

         mov  ax,DGROUP                ;put data segment address
         mov  ds,ax                    ;  into data segment
         mov  es,ax                    ;  and extra segment

         call _hndirq                  ;invoke "C" interrupt handler

         mov  dx,io_eoi                ;get 8259 OCW2 I/O address in DX
         mov  al,eoi                   ;indicate End Of Interrupt
         out  dx,al                    ;  to interrupt controller
         jmp  short $+2
         jmp  short $+2
         cmp  dx,0a0h                  ;is this master 8259?
         jb   irq90                    ;  yes - jump to skip second EOI
         sub  dx,080h                  ;point to master 8259
         out  dx,al                    ;write EOI to interrupt controller
         jmp  short $+2
         jmp  short $+2


irq90:
         pop  es                       ; recover registers
         pop  ds
         pop  di
         pop  si
         pop  bp
         pop  dx
         pop  cx
         pop  bx
         pop  ax
         iret
irq      endp


_TEXT    ENDS
         END
